from typing import Optional, Dict

from moviebotapi import Session
from moviebotapi.core import utils
from moviebotapi.core.models import MediaType


class MediaImage:
    """影片图片资源"""
    source: str
    banner: str
    poster: str
    small_poster: str
    clear_logo: str
    background: str
    small_backdrop: str
    thumb: str
    main_background: str
    main_poster: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class ScraperApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def get_images(self, media_type: MediaType, tmdb_id: int, season_number: Optional[int] = None,
                   episode_number: Optional[int] = None) -> Optional[MediaImage]:
        data = self._session.get('scraper.get_media_image', {
            'media_type': media_type.value,
            'tmdb_id': tmdb_id,
            'season_number': season_number,
            'episode_number': episode_number
        })
        if not data:
            return
        return MediaImage(data)



import datetime
import json
from typing import Dict, List, _GenericAlias

from moviebotapi.core.models import MediaType


def copy_value(source: Dict, target: object) -> None:
    if not source:
        return
    if not target or not target.__annotations__:
        return
    for name in target.__annotations__:
        anno = target.__annotations__[name]
        setattr(target, name, parse_value(anno, source.get(name)))


def _list_value(value):
    if isinstance(value, str):
        if value[0] in ['{', '[']:
            return json.loads(value)
        else:
            return value.split(',')
    else:
        return list(value)


def _dict_value(value):
    if isinstance(value, str):
        return json.loads(value)
    else:
        return dict(value)


def parse_value(func, value):
    if value is not None:
        if func == bool:
            if value in (1, True, "1", "true"):
                return True
            elif value in (0, False, "0", "false"):
                return False
            else:
                raise ValueError(value)

        elif func in (int, float):
            try:
                return func(value)
            except ValueError:
                return float('nan')
        elif func == datetime.datetime:
            return datetime.datetime.strptime(value, '%Y-%m-%d %H:%M:%S')
        elif func in [Dict, dict]:
            return _dict_value
        elif func in [List, list]:
            return _list_value(value)
        elif func == MediaType:
            l = value.lower()
            if l == 'movie':
                return MediaType.Movie
            if l in ['tv', 'series']:
                return MediaType.TV
        elif isinstance(func, _GenericAlias):
            if func.__origin__ in [List, list]:
                list_ = _list_value(value)
                res = []
                for x in list_:
                    res.append(parse_value(func.__args__[0], x))
                return res
        return func(value)
    return value



from typing import Dict, List, Optional

from moviebotapi import Session
from moviebotapi.core import utils
from moviebotapi.core.models import MediaType
from moviebotapi.subscribe import SubStatus


class DoubanPeople:
    """剧组人员、导演、演员等人员信息"""
    douban_id: int
    tmdb_id: int
    imdb_id: str
    name: str
    en_name: str
    role: str
    pic_url: str
    type: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self)


class DoubanMedia:
    id: int
    url: str
    release_year: str
    local_name: str
    en_name: str
    cover_image: str
    rating: float
    user_id: str
    intro: str
    director: List[DoubanPeople]
    actor: List[DoubanPeople]
    duration: int
    trailer_video_url: str

    def __init__(self, data: Dict):
        utils.copy_value(data, self)
        # 对历史遗留接口不规范命名做更正
        self.media_type: MediaType = utils.parse_value(MediaType, data.get('type'))
        self.aka_names: List[str] = utils.parse_value(List[str], data.get('alias'))
        self.genres: List[str] = utils.parse_value(List[str], data.get('cates'))
        self.country: List[str] = utils.parse_value(List[str], data.get('area'))
        self.cn_name: str = utils.parse_value(str, data.get('name'))
        self.premiere_date: str = utils.parse_value(str, data.get('release_date'))
        self.imdb_id: str = utils.parse_value(str, data.get('imdb'))
        self.episode_count: int = utils.parse_value(int, data.get('total_ep_count'))
        self.season_index: int = utils.parse_value(int, data.get('season_index'))


class DoubanSearchResult:
    id: int
    cn_name: str
    rating: float
    url: str
    app_url: str
    sub_id: int
    status: SubStatus

    def __init__(self, data: Dict):
        utils.copy_value(data, self)
        self.poster_url: str = utils.parse_value(str, data.get('poster_path'))


class DoubanApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def get(self, douban_id: int) -> Optional[DoubanMedia]:
        meta = self._session.get('douban.get', {
            'douban_id': douban_id
        })
        if not meta:
            return
        return DoubanMedia(meta)

    def search(self, keyword: str) -> List[DoubanSearchResult]:
        result = self._session.get('movie.search_douban', {
            'keyword': keyword
        })
        if not result:
            return []
        return [DoubanSearchResult(x) for x in result]



import datetime
from enum import Enum
from typing import Dict

from moviebotapi import Session
from moviebotapi.core import utils


class TrafficManagementStatus(int, Enum):
    Disabled = 0
    Initiative = 1
    Passive = 2


class SiteStatus(int, Enum):
    Normal = 1
    Error = 2


class Site:
    gmt_modified: datetime.datetime
    uid: int
    username: str
    cookie: str
    web_search: bool
    smart_download: bool
    share_rate: float
    upload_size: float
    download_size: float
    is_vip: bool
    status: SiteStatus
    traffic_management_status: TrafficManagementStatus
    # 主动模式时上传流量目标，单位GB
    upload_kpi: int
    proxies: str
    user_agent: str

    def __init__(self, data: Dict, api: "SiteApi"):
        utils.copy_value(data, self)
        self.site_id: str = utils.parse_value(str, data.get('site_name'))
        self.site_name: str = utils.parse_value(str, data.get('alias'))
        self._api = api


class SiteApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def list(self):
        list_ = self._session.get('site.get_sites')
        if not list_:
            return
        return [Site(x, self) for x in list_]



import datetime
from enum import Enum
from typing import Dict, Optional

from moviebotapi import Session
from moviebotapi.core import utils
from moviebotapi.core.models import MediaType


class SubStatus(int, Enum):
    """
    订阅中
    """
    Subscribing = 0
    """
    订阅已经完成
    """
    Completed = 1
    """
    等待更好的版本
    """
    WaitVersion = 2


class Subscribe:
    id: int
    uid: int
    en_name: str
    desc: str
    rating: float
    status: SubStatus
    from_action: str
    alias: str
    poster_path: str
    download_mode: int
    episode_count: int
    release_year: int
    season_year: int
    remote_search: bool
    is_aired: bool
    media_type: MediaType
    thumb_image_path: str
    douban_id: int
    tmdb_id: int
    imdb_id: str
    genres: str
    area: str
    choose_rule_name: str
    release_date: str
    cn_name: str
    priority_keywords: str
    filter_config: Dict
    gmt_create: datetime.datetime
    gmt_modified: datetime.datetime

    def __init__(self, data: Dict, api: 'SubscribeApi'):
        utils.copy_value(data, self)
        self._api = api
        self.season_number: int = utils.parse_value(int, data.get('season_index'))

    def delete(self, deep_delete: bool = False):
        self._api.delete(self.id, deep_delete)


class SubscribeApi:
    def __init__(self, session: Session):
        self._session: Session = session

    def get(self, subscribe_id: int) -> Optional[Subscribe]:
        sub = self._session.get('subscribe.get_sub', params={'id': subscribe_id})
        if not sub:
            return
        return Subscribe(sub, self)

    def list(self, media_type: Optional[MediaType] = None, status: Optional[SubStatus] = None):
        params = {}
        if media_type:
            params.update({'media_type': media_type.value})
        if status is not None:
            params.update({'status': status.value})
        list_ = self._session.get('subscribe.list', params=params)
        if not list_:
            return []
        return [Subscribe(x, self) for x in list_]

    def delete(self, subscribe_id: int, deep_delete: bool = False):
        self._session.post('subscribe.delete_sub', json={
            'id': subscribe_id,
            'deep_delete': deep_delete
        })

    def sub_by_tmdb(self, media_type: MediaType, tmdb_id: int, season_number: Optional[int] = None):
        params = {}
        if media_type:
            params.update({'media_type': media_type.value})
        if tmdb_id:
            params.update({'tmdb_id': tmdb_id})
        if season_number is not None:
            params.update({'season_number': season_number})
        self._session.get('subscribe.sub_tmdb', params=params)

    def sub_by_douban(self, douban_id: int, filter_name: Optional[str] = None):
        params = {'id': douban_id}
        if filter_name:
            params.update({'filter_name': filter_name})
        self._session.get('subscribe.sub_douban', params=params)



from moviebotapi.core.session import Session
from moviebotapi.douban import DoubanApi
from moviebotapi.scraper import ScraperApi
from moviebotapi.site import SiteApi
from moviebotapi.subscribe import SubscribeApi
from moviebotapi.tmdb import TmdbApi
from moviebotapi.user import UserApi


class MovieBotServer:
    def __init__(self, session: Session):
        self.user = UserApi(session)
        self.subscribe = SubscribeApi(session)
        self.scraper = ScraperApi(session)
        self.douban = DoubanApi(session)
        self.tmdb = TmdbApi(session)
        self.site = SiteApi(session)



# -*- coding: utf-8 -*-
from setuptools import setup

# Get README.rst contents
with open('README.md', 'r', encoding="utf-8") as f:
    readme = f.read()
requirements = []
with open('requirements.txt') as handle:
    for line in handle.readlines():
        if not line.startswith('#'):
            package = line.strip().split('=', 1)[0]
            requirements.append(package)
setup(
    name='movie-bot-api',
    version='0.0.2',
    author='yee',
    author_email='yipengfei329@gmail.com',
    url='https://github.com/pofey/movie-bot-api',
    description='智能影音机器人MovieBot的接口SDK',
    python_requires='>=3.8',
    long_description=readme,
    long_description_content_type="text/markdown",
    keywords=['movie bot', 'movie robot'],
    packages=['moviebotapi','moviebotapi.core'],
    install_requires=requirements
)